#pragma strict

var ColorBack		: Color;
var ColorOn			: Color;
var ColorOff		: Color;
var MaterialInOn	: Material;
var MaterialInOff	: Material;
var MaterialOut		: Material;
var RotateEffect	: Vector3 = Vector3.zero;

private var lineWidth			: float = 0.05;
private var originalScale		: Vector3;
private var originalRotation	: Quaternion;
private var checkpoints			: float[];

// Scripts
@System.NonSerialized var GAME 			: scr_GAME;
@System.NonSerialized var OBJ_OUT	 	: GameObject;
@System.NonSerialized var RENDERER 		: Renderer;
@System.NonSerialized var RENDERER_OUT 	: Renderer;

function Awake () {
	Debug.Log("OBJ Awake >> "+name);
	GAME = GameObject.Find("GAME").GetComponent(scr_GAME);
	RENDERER = gameObject.renderer;
	originalScale = gameObject.transform.localScale;
	originalRotation = gameObject.transform.rotation;
	makeCheckpoints();
}


function Start () {
	Debug.Log("OBJ Start >> "+name);
	OBJ_OUT = Instantiate(gameObject, transform.position, transform.rotation);
	OBJ_OUT.Destroy( OBJ_OUT.GetComponent(scr_OBJ) );
	OBJ_OUT.gameObject.name = name+"_out";
	OBJ_OUT.transform.parent = transform;
	RENDERER_OUT = OBJ_OUT.GetComponent(Renderer);
	RENDERER_OUT.transform.localScale = Vector3.one + Vector3( 1.0/originalScale.x, 1.0/originalScale.y, 1.0/originalScale.y ) * lineWidth;
	RENDERER_OUT.material = MaterialOut;
}

function Update () {
	// resize screen -- KEEP ASPECT
	//var aspect : float = (Screen.width / parseFloat(Screen.height));
	//gameObject.transform.localScale = Vector3.Scale( originalScale, Vector3( aspect/1.777, aspect, 1.0) );

	// paint OBJ
 	if ( GAME.state <= GameState.Ready )
		RENDERER.material = MaterialInOn;
	else
		RENDERER.material = ( IsON() ? MaterialInOn : MaterialInOff );
	var c : Color = RENDERER.material.color;
	var invColor : Color = Color( 1.0 - c.r, 1.0 - c.g, 1.0 - c.b );
	MaterialOut.color = invColor;
	MaterialInOn.color = ColorOn;
	if ( c == getColorBack() )
	{
		MaterialOut.color = ColorOn;
		RENDERER_OUT.enabled = true;
	}
	else
		RENDERER_OUT.enabled = false;
		
	
	// react to HITTING
	if ( ! GAME.HITTING )
	{
		var dx : float = Random.Range(-1.0, 1.0) * GAME.NOISE;
		var dy : float = Random.Range(-1.0, 1.0) * GAME.NOISE;
		transform.position.x = dx;
		transform.position.y = dy;
		var a : float = 1440 * (1.0 - Mathf.Cos(Mathf.PI*0.5*GAME.PROG));
		if (RotateEffect.x || RotateEffect.y || RotateEffect.z)
		{
			var totalRot : Quaternion = Quaternion.Euler( RotateEffect.x * a, RotateEffect.y * a, RotateEffect.z * a  );
			transform.rotation = Quaternion.Lerp( originalRotation, totalRot, GAME.NOISE );
		}
		else
			transform.rotation = originalRotation;
	}
	else
	{
		transform.position *= 0.66;
		transform.rotation = originalRotation;
	}
}







function IsON() : boolean
{
	if (GAME.PROG > 1.0)
		return true;
	
	//return ( parseInt(GAME.timeElapsed) % 2 == 1 );
	
	var count : int = 0;
	for (var t in checkpoints)
	{
		if ( t > GAME.timeElapsed)
			break;
		count++;
	}
	return ( count % 2 == 1 );
}

function getColorBack() : Color
{
 	if ( GAME.state <= GameState.Ready )
		//return Color.black;
		return ColorBack;
	else if ( GAME.state <= GameState.Playing && ! GAME.HITTING )
		return ColorBack;
	else
		return Color.black;
}





function makeCheckpoints()
{
	var ar = new Array();
	
	//
	// Make Checkpoints
	var n : float;
	var count : float;
	var inc : float;
	var l : int = parseInt( name.Substring(3,2) );
	switch ( l )
	{
		case 1:
		case 4:
		case 7:
			// Each 2 seconds, decreasing
			count = 0;
			for (n = 2.0 ; n < GAME.SECONDS ; n += inc)
			{
				ar.Add( n );
				inc = LIB.LMap( n, 2.0, GAME.SECONDS, 2.0, 0.5 );
				// reduce in halfe very 3...
				if ( l  > 1 && count % 3 == 2)
					inc *= 0.5;
				count ++;
			}
			break;
		case 2:
		case 6:
		case 8:
			// Each 1 second, decreasing
			for (n = 1.0 ; n < GAME.SECONDS ; n += inc)
			{
				ar.Add( n );
				inc = LIB.LMap( n, 1.0, GAME.SECONDS, 1.0, 0.5 );
			}
			break;
		case 3:
		case 5:
		case 9:
		case 10:
			// Random, decreasing
			var max : float = ( l < 9 ? 4.0 : 2.0 );
			for (n = 1.0 ; n < GAME.SECONDS ; n += inc)
			{
				ar.Add( n );
				max = LIB.LMap( n, 1.0, GAME.SECONDS, 4.0, 1.0 );
				inc = Random.Range( 0.5, max );
			}
			break;
	}
		
	//
	// SAVE CHECKPOINTS
	//for (var f in checkpoints)
	//	Debug.Log("CHECKPOINT = "+f);
	checkpoints = ar.ToBuiltin(float);
}

